/*******************************************************************************
 *
 *
 *	Copyright 2003 Steven James <pyro@linuxlabs.com> and
 *	LinuxLabs http://www.linuxlabs.com
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include <etherboot.h>
#include <pci.h>
#include <timer.h>
#include <lib.h>
        
#define DEBUG_THIS DEBUG_USB
#include <debug.h>

#define DPRINTF debug

#include "hci.h"
#include "debug_x.h"

struct usbdisk_info_t {
        struct controller *ctrl;
        uint16_t heads;
        uint16_t cylinders;
        uint16_t sectors_per_track;
        uint8_t  model_number[41];
        uint8_t  slave;
        sector_t sectors;
        int  address_mode;
#define ADDRESS_MODE_CHS    0
#define ADDRESS_MODE_LBA    1
#define ADDRESS_MODE_LBA48  2
#define ADDRESS_MODE_PACKET 3
        uint32_t hw_sector_size;
        unsigned drive_exists : 1;
        unsigned slave_absent : 1;
        unsigned removable : 1;
	
	unsigned char usb_device_address;	
};

struct usbdisk_info_t usbdisk_info;

int usb_probe(int drive)
{
	struct usbdisk_info_t *info = &usbdisk_info;

	unsigned char value;

        int i,res;
        int error_count=100;
        unsigned char sense_data[32];


	printf("LinuxLabs USB bootloader\n");

//	outb( 0x30, 0x70);	// reset primary boot
//	outb( 0xff, 0x71);
	init_devices();
	hci_init();

	info->usb_device_address = 0;
        // find first usb device

        while(error_count && (res = poll_usb()))        // keep polling usb until no more devices are enumerated
                if(res<0)
                        if(!--error_count)
                                printf("There is a USB device, but it won't init! This is a bad thing.\n");

        for(i=0; i< next_usb_dev ; i++) {
                if(usb_device[i].class == 0x08 && usb_device[i].subclass == 0x06 && usb_device[i].protocol == 0x50) {
                        printf("Found USB block device %d\n", i);
			if(drive==0) {
                        	info->usb_device_address = i;
				break;
			}
			drive--;
                }
        }
	
	if(info->usb_device_address == 0) return -1;

	return 0;
}
int usb_read(int drive, sector_t sector, void *buffer)
{
        struct usbdisk_info_t *info = &usbdisk_info;
        int result;
	
	result = ll_read_block(info->usb_device_address, buffer,sector, 1);
#if 0
        //debug("drive=%d, sector=%ld\n",drive,(unsigned long) sector);
        /* Report the buffer is empty */
        if (sector > info->sectors) {
                return -1;
        }
        if (info->address_mode == ADDRESS_MODE_CHS) {
                result = ide_read_sector_chs(info, buffer, sector);
        }
        else if (info->address_mode == ADDRESS_MODE_LBA) {
                result = ide_read_sector_lba(info, buffer, sector);
        }
        else if (info->address_mode == ADDRESS_MODE_LBA48) {
                result = ide_read_sector_lba48(info, buffer, sector);
        }
        else if (info->address_mode == ADDRESS_MODE_PACKET) {
                result = ide_read_sector_packet(info, buffer, sector);
        }
        else {
                result = -1;
        }
#endif
        return result;
}
 
